/* INETLOG.CMD
 1995 by Jerry Levy,  Marblehead, MA (USA) 03 Nov 95
jlevy@ibm.net
 1996, 1997, 1998 by Chuck McKinnis,  Sandia Park, NM (USA) 01 Feb 98
mckinnis@ibm.net

REXX Program to extract and totalize daily and monthly time-ons by
analyzing the IBM WARP Internet Dialer log file */

Trace 'N'
version = '5.4'
what_r_we = 'INETLOG  v'||version ' 1995 - 1998 by Jerry Levy and Chuck McKinnis'
Parse Upper Arg otherparms

/* Right to freely use, modify, distribute granted but please acknowledge
   source as a courtesy if you build upon it.  Please pass on comments,
   suggestions, problems to jlevy@ibm.net or chmckin@ibm.net
   ------------------------------------
   Getting Started (See ReadMe.1st)
   Program History (See changes.txt)
   Use of Command-line options:  (See ReadMe.1st)
   ----------------------------------- */

/* The following are acceptable command-line parameters for special runs.
    Only one of these can be used in a run and case is ignored.

      CONDENSE to remove extraneous lines from connection log file

      EOF2CRLF to replace any EOF characters added by using an editor 
               against the log file

   The following are acceptable command-line parameters (case is ignored 
   and only the first letter is required):

   These parameters can be overridden on a run by run basis only.

      L<ogFileName>=connection log file name, default is the log file
                    pointed to by DIALER.INI (or other specified ini file).

      Q<uiet> run without output to screen, default is to output results

   The following parameters may be overridden permanently with a command 
   line parameters and will be stored in INETLOG.INI.

      I<niFileName>=ini file name of dialer, default is dialer.ini

      O<utputFileName>=output log file name, default is inetlog.log

      S<ummaryFileName>=summary log file name, default is inetsum.log

      DataPath=path to output log and summary log, default is the install path

      D<isplayDailyReport>=display daily report? (Yes or No), default is No

      P<auseAsScreenFills>=pause as each screen fills up? (Yes or No), default is Yes

      W<arnPercent>=percentage for warning on connect.log, default is 85

   */

Call RxFuncAdd 'SysLoadFuncs', 'RexxUtil', 'SysLoadFuncs'
Call SysLoadFuncs

Signal On Failure Name ErrHandler
Signal On Halt Name ErrHandler
Signal On Syntax Name ErrHandler
/* Signal On Error Name ErrHandler */  /* we don't use this one */

/*========MAIN PROGRAM=========*/
Call SysCls

/* Where are we ? */
Parse Source . . install_path .
install_path = Filespec('D',install_path) || Filespec('P',install_path)
ini_file = install_path || 'INETLOG.INI'
save_path = Directory()
our_path = Directory(install_path)

Parse Value Initialize_ini(what_r_we, version) With tcpip_etc_path dialer_log_file dialer_log_size install_path data_path

Call Initialize_parms

If ModifyLogFile = 'TESTFIXEDLOG' Then Do
   Say 'Not permitted to use TestFixedLog as a command-line argument.  Exiting...'
   Call Cleanup
   End

If Translate(log_file) <> Translate(dialer_log_file) Then Do
   Say 'You have chosen to analyze' log_file
   Say 'Your dialer log file is' dialer_log_file
   Say 'If you want to quit now, press ESC.'
   answer = Say_message('Otherwise, press any other key to continue.')
   If answer = esc Then Call Cleanup
   End

If ModifyLogFile = 'CONDENSE' Then Call Condense_log
If ModifyLogFile = 'EOF2CRLF' Then Call Fix_eof

Call Analyze_logfile /* gets, calculates, totalizes connect times.
                        Mostly a bunch of conditionals with a Call to a 
                        data-formatting routine. All is stored in
                        variables for output all at once */

Call Outputter  /* Outputs everything to console and to disk */

If ModifyLogFile = 'TESTFIXEDLOG' Then Call Log_file_cleanup /* rename, etc., modified log file */

Call Cleanup  /* Exits */
/*====END OF MAIN PROGRAM=======*/
Analyze_logfile:
   /* Now find all lines in connect.log that contain the key_phrase 
      string.  A typical line generated in the connect.log after 
      disconnect is either this for a dialer version preceding v. 1.45:
	11/15 19:08:38 Disconnected after 00:06:20  0 errors  0 discards
      or this style for v. 1.45 and later:
	1995/11/15 19:08:38 Disconnected after 00:06:20  0 errors  0 discards
      which we would parse as follows:
	date  word2       word3    word4 connect_time */

   Call SysFileSearch key_phrase, log_file, 'line.', 'N'

   Call SysFileSearch user_phrase, log_file, 'user.', 'N'

   /* this section will read backwards through the connection log file
      data and attempt to assign account information and userid to each
      connection log record */
   k = user.0
   Do i = line.0 To 1 By -1 While k > 0
      Parse Var line.i disc_line_no .
      acct_data = 'acctid=unknown userid=unknown'
      j = i - 1
      If j > 0 Then Parse Var line.j prev_disc_line_no .
      Else prev_disc_line_no = 0
      If k > 0 Then Do
         Parse Var user.k user_line_no . . acctid userid .
         If disc_line_no > user_line_no & user_line_no > prev_disc_line_no Then Do
            acct_data = 'acctid=' || acctid 'userid=' || userid
            k = k - 1
            End
         End
      line.i = line.i acct_data
      End

   rc = 0

   Do i = 1 To line.0
      Parse Var line.i . date word2 word3 word4 connect_time remainder
      If remainder = '' Then Iterate i
      Parse Var remainder . 'acctid=' acctid 'userid=' userid .
      acctid = Strip(acctid)
      userid = Strip(userid)
      /* Date, excluding year if year is or is not present, is 5 chars: mm/dd */
      date2 = Right(date,5)
      /* Make all dates uniform.  If more than mm/dd (5 chars) then year 
         is there.  The length(date) - 6 = length of however many
         characters are used for the year (4 now (e.g., 1995) but some
         joker might change it to 2 (e.g., 95) We then add 1 char for the
         / separator + 1 */
      If Length(date) > 5 Then year = Substr((date), 1, Length(date) -6 ) || ' '
      Else  year = '     '  /* If pre v1.45 Dialer, no year, pad 5 spaces */
      mm = Substr(date2, 1, 2)                  /* Extract the month of a connection as a 2-dig number*/
      dd = Substr(date2, 4, 2)                  /* ...and the day */
      Parse Var word2 t_hr ':' t_min ':' t_sec  /* Extract for time stamp and save */
      If year <> '     ' Then t_yr = Strip(year)
      Else t_yr = '    '
      time_stamp = 'T' || t_yr || mm || dd || t_hr || t_min || t_sec
      hrs = Substr(connect_time, 1, 2)          /* Extract the number of hours on-line */
      mins = Substr(connect_time, 4, 2)         /*... and mins */
      secs = Substr(connect_time, 7, 2)         /*...and seconds */
      If hrs < 0,                               /* If hrs is negative */
         | mins < 0,                            /* or mins is negative */
         |If secs < 0 Then Call ErrDST          /* or secs is negative.  Time change error? Abort */

      time_on = (60*hrs + mins + (1/60)*secs)  /* Calculate time_on for that connection */
      If time_stamp > last_time_stamp Then Call Monthly_update

      IF old_d = 0 Then Do        /* for very first connection line*/
         old_m = mm
         old_d = dd
         old_y = year
         signons_each_day = 1     /* reset to 1 */
         signons_each_month = 1
         daily_time_on = time_on  /* This and next one are timeons in minutes */
         monthly_time_on = time_on
         End

      Else  /* continue to accumulate times if same month and day */
         If old_m = mm & old_d = dd & Month <> 0 Then Do
            old_y = year
            signons_each_day = signons_each_day + 1
            signons_each_month = signons_each_month + 1
            daily_time_on = daily_time_on + time_on
            monthly_time_on = monthly_time_on + time_on
            End

         Else  /* new day of same month */
            If old_m = mm & old_d <> dd Then Do
               Call Prepare_data
               dcounter = dcounter + 1
               year.dcounter = year
               year.mcounter = year
               dayline.dcounter = year.dcounter||month.old_m old_d d_signons d_mins d_hhmmss d_hrs

               old_d = dd
               old_y = year
               signons_each_day = 1  /* Start counting over again */
               signons_each_month = signons_each_month + 1
               daily_time_on = time_on
               monthly_time_on = monthly_time_on + time_on
               End

            Else  /* for any new month, which by definition is also a new day */
               If old_m <> mm & old_m <> 0 Then Do
                  Call Prepare_data
                  dcounter = dcounter + 1
                  year.dcounter = old_y
                  dayline.dcounter = year.dcounter||month.old_m old_d d_signons d_mins d_hhmmss d_hrs

                  mcounter = mcounter + 1
                  year.mcounter = old_y
                  monthline.mcounter = year.mcounter||month.old_m m_signons m_mins m_hhmmss m_hrs

                  old_m = mm
                  old_d = dd
                  old_y = year
                  signons_each_day = 1
                  signons_each_month = 1
                  daily_time_on = time_on
                  monthly_time_on = time_on
                  End

      End /* end of all these If's and Else If's of searching for 
             key_phrase in all possible lines in the connect.log */

   /* Now, since last day and last month is done: */
   Call Prepare_data
   dcounter = dcounter + 1
   year.dcounter = year
   dayline.dcounter = year.dcounter || month.old_m old_d d_signons d_mins d_hhmmss d_hrs

   mcounter = mcounter + 1
   year.mcounter = year
   monthline.mcounter = year.mcounter || month.old_m m_signons m_mins m_hhmmss m_hrs
   Call SysIni ini_file, 'INETLOG', 'LastTimeStamp', time_stamp  /* save the last */
   Return	/* from Analyze_logfile	*/

Monthly_update:
   s_yr = Strip(t_yr)
   s_mo = Strip(mm)
   If summary.acctid.userid.s_yr.s_mo <> '' Then Do
      Parse Var summary.acctid.userid.s_yr.s_mo s_yr s_mo s_stamp s_sess s_min .
      If time_stamp > s_stamp Then Do
         s_sess = s_sess + 1
         s_min = s_min + time_on
         summary.acctid.userid.s_yr.s_mo = s_yr s_mo time_stamp s_sess s_min
         End
      End
   Else Do
      x = monthly.0 + 1
      monthly.x = acctid userid s_yr s_mo
      monthly.0 = x 
      summary.acctid.userid.s_yr.s_mo = s_yr s_mo time_stamp '1' time_on
      End
   Return

Outputter:
   /* Now output everything to console and to file */

   /* get the screen size; rows is what we are interested in */
   Parse Value SysTextScreenSize() With rows cols
   Call Stream STDIN, 'C', 'OPEN READ'
   /* Tell us all */
   intro = what_r_we
   Say intro
   If otherparms <> '' Then Do
      Say crlf || 'Running with parms =' otherparms
      Call Lineout output_file crlf || 'Running with parms =' otherparms
      End
   Call Lineout output_file, intro
   intro = crlf || 'Analysis of' log_file '(' || Date() '@' Time() || ')'
   Say intro
   Call Lineout output_file, intro
   If daily_report Then Call Say_out crlf || DAILY TOTALS
   Call Lineout output_file, crlf || DAILY TOTALS
   Do j = 1 To dcounter
      If daily_report Then Call Say_out dayline.j
      Call Lineout output_file, dayline.j
      If one_screen_atatime & daily_report & \ quiet Then			
         If (j - rows + 6) // (rows - 5) = 0 then
            answer = Say_message('               Press any key to continue...')
      End

   If one_screen_atatime & daily_report & \ quiet Then			
      If (j - rows + 6) // (rows - 5) > 5 Then Do  /* If at about 5 lines to end */
         answer = Say_message('   Daily Totals done, press any key for Monthlies...')
         j = 0	/* If decision is to do a 'Press-Any-Key to continue', reset to 0 */
         End
   If \ daily_report Then j = 0

   /* back up the summary file and delete it */
   If File_cmd(summary_file,'X') Then Do
      Parse Var summary_file fname '.' ext
      '@COPY' summary_file fname || '.bak > NUL'
      Call SysFileDelete summary_file
      End
   rc = File_cmd(summary_file,'W')
   acct_user_save = ''
   Do k = 1 To monthly.0
      If Abbrev(monthly.k,'*') Then Do  /* write comments back out */
         Call Lineout summary_file, monthly.k
         Iterate k
         End
      Parse Var monthly.k acctid userid s_yr s_mo
      acct_user = acctid userid
      Call Lineout summary_file, acctid userid '*' summary.acctid.userid.s_yr.s_mo
      Parse Var summary.acctid.userid.s_yr.s_mo . . . s_sess s_mins
      m_sess = Format(s_sess, sig_x) || 'X'
      m_mins = Format(s_mins, sig_min, 2) 'mins'
      mo_hh = Format(Trunc(s_mins / 60),sig_hr)
      mo_mm = Right(Trunc(s_mins // 60),2,'0')
      mo_ss = Right(Trunc(60 * ((s_mins // 60) - Trunc(s_mins // 60))),2,'0') 
      m_hhmmss = '  '||mo_hh||':'||mo_mm||':'||mo_ss
      m_hrs = format((s_mins / 60), sig_hr, 2) 'hrs'
      a_sess = s_mins / s_sess 
      If a_sess < 60 Then a_sess = Format(a_sess,2,0) 'mins'            
      Else a_sess = Format((a_sess / 60),2,2) 'hrs'              
      a_sess = '- Ave =' a_sess
      monthline = s_yr month.s_mo m_sess m_mins m_hhmmss m_hrs a_sess
      If acct_user <> acct_user_save Then Do
         intro = crlf || 'MONTHLY TOTALS for Account(' || acctid || ') Userid(' || userid || '}'
         Call Say_out intro
         Call Lineout output_file, intro
         acct_user_save = acct_user
         End
      Call Lineout output_file, monthline
      Call Say_out monthline
      If one_screen_atatime & \ quiet Then
         If (k + j - rows + 6) // (rows - 6) = 0 Then
            answer = Say_message('               Press any key to continue...')
      End
   rc = File_cmd(summary_file,'C')
   finished = crlf || 'End of analysis of' log_file
   Say finished
   Call Lineout output_file, finished

   Return  /* from Outputter */

Prepare_data:
   /* Calculates and formats what is to be put into an output line */
 
   /* Signons per day or month, as, e.g.: '4X' */
   d_signons = Format(signons_each_day, sig_x)||'X'
   m_signons = Format(signons_each_month, sig_x)||'X'

   /* Minutes/day, minutes/month, hrs/day, hrs/month as, e.g.: '105.50 mins or hrs' */
   d_mins = Format(daily_time_on, sig_min, 2) 'mins'
   d_hrs = Format((daily_time_on/60), sig_hr, 2) 'hrs'
   m_mins = Format(monthly_time_on, sig_min, 2) 'mins'
   m_hrs = Format((monthly_time_on/60), sig_hr, 2) 'hrs'

   /* minutes, seconds per day or month, as 2-digit numbers; hrs can exceed 2 digs */  
   dy_hh = Format(Trunc(daily_time_on/60),sig_hr)			
   dy_mm = Right(Trunc(daily_time_on//60),2,'0')
   dy_ss = Right(Trunc(60*((daily_time_on//60) - Trunc(daily_time_on//60))),2,'0')
   mo_hh = Format(Trunc(monthly_time_on/60),sig_hr)
   mo_mm = Right(Trunc(monthly_time_on//60),2,'0')
   mo_ss = Right(Trunc(60*((monthly_time_on//60) - Trunc(monthly_time_on//60))),2,'0') 

   /* hours, minutes, seconds per day or month, as, e.g.: '1:45:30'*/  
   d_hhmmss = '  '||dy_hh||':'||dy_mm||':'||dy_ss
   m_hhmmss = '  '||mo_hh||':'||mo_mm||':'||mo_ss

   Return /* Prepare_data */

Condense_log:
   trace_condense_log = Trace()
   Trace 'N'
   log_file_bak = log_file  /* Save this filespec */
   /* first we check to see if the summary file is up to date */
   rc = SysFileTree(summary_file,'file.','FT')
   If rc = 0 & file.0 > 0 Then Do
      Parse Var file.1 summary_stamp .
      rc = SysFileTree(log_file,'file.','FT')
      Parse Var file.1 log_stamp .
      If log_stamp > summary_stamp Then Do
         Say 'The timestamp on the log file' log_file 'is later than the'
         Say 'timestamp on the summary file' summary_file || '.'
         Say 'Please run INETLOG without CONDENSE to update the summary file.'
         Call Cleanup
         End
      End
   Call SysFileSearch user_phrase, log_file, 'user.', 'N'
   l = 1
   Call SysFileSearch key_phrase, log_file, 'line.', 'N'
   rc = 0
   Do i = 1 To line.0
      Parse Var line.i line_num line_out
      j = i - 1
      If j > 0 Then Parse Var line.j prev_line_num .
      Else prev_line_num = 0
      Do k = l To user.0 While l < user.0 | l = user.0
         Parse Var user.k user_num user_out
         If line_num > user_num & user_num > prev_line_num Then Do
            Call Lineout output_file, user_out
            If l < user.0 | l = user.0 Then l = k + 1
            Leave k
            End   
         End
      Call Lineout output_file, line_out
      End
   Call File_cmd log_file, 'C'
   log_file_size = Chars(log_file)
   Call File_cmd log_file, 'C'
   Call File_cmd output_file, 'C'
   output_file_size = Chars(output_file)
   Call File_cmd output_file, 'C'
   log_file = output_file          /* Use output_file that we just created from Fix_Log as the log file */
   output_file = output_file_bak   /* Restore this as file to output to */
   ModifyLogFile = 'TESTFIXEDLOG'  /* Run the main routine to do test new connection log */
   Call SysCls
   Say 'The conversion is done.'
   Say 'The input log file had' log_file_size 'bytes.'
   Say 'The new log file has' output_file_size 'bytes.'
   Say crlf
   Say log_file 'is the condensed connection log file.'
   Say 'We will now test it.  Please evaluate whether'
   Say 'the data being spit out is reasonable.  The output will'
   Say 'also be saved to disk as' output_file
   Say 'which can be examined with the System Editor.'
   Say 'If you want to quit now, press ESCAPE.'
   If Say_message('Otherwise, press any other key to run the test.') = esc Then Call Cleanup
   Call SysCls
   Say 'Starting the analysis of' log_file 'as the log file.'
   Call SysSleep 2  /* Let us read the message */

   /* Back up any output file of the same name if it exists, then erase orig. */
   If File_cmd(output_file, 'X') Then Do
      Parse Var output_file fname '.' ext
      '@COPY' output_file fname||'.bak > NUL'
      Call SysFileDelete output_file
      End
   Trace (trace_condense_log)
   Return  /* from Condense_log */

ErrDST:  /* If error due to time change */
   Say 'INETLOG has found a negative number for time-on-line'
   Say 'in an entry in your' log_file 'file.'
   Say ''
   Say 'You may have reset your clock backwards while on line, e.g.,'
   Say 'while changing from Summer time (Daylight Saving Time) to'
   Say 'Winter time (Standard time).'
   Say ''
   Say 'To fix:'
   Say '   1.  Open' log_file 'in your System Editor'
   Say '   2.  Edit any connection time(s) that have a minus sign.'
   Say '       Example (for a typical one-hour summer-to-winter time correction):'
   Say ''
   Say '       change'
   Say '          1995/10/29 06:26:43 Disconnected after -00:-58:-4  0 errors  0 discards'
   Say '             to'
   Say '          1995/10/29 06:26:43 Disconnected after 00:01:56  0 errors  0 discards'
   Say ''
   Say '       If the error was not from a summer-time change, make whatever'
   Say '       correction seems reasonable to eliminate the offending minus'
   Say '       signs (like just removing them).'
   Say ''
   Say '   3.  Save' log_file
   Say '   4.  Run INETLOG with the EOF2CRLF parameter to remove the EOF characters'
   Say '       added by the editor.'
   Say 'Aborting . . .'
   Call Cleanup
   Return

ErrHandler:
   Call Beep 300, 500
   Say 'Rexx error' rc 'in line' sigl||':' errortext(rc)
   Say sourceline(sigl)
   Call Cleanup
   Return

Cleanup:  /* Exit */
   rc = SysFileTree(dialer_log_file, 'info.', 'F')
   Parse Var info.1 . . log_file_size .
   If dialer_log_size > 0 Then Do
      log_file_pct = Format((log_file_size / dialer_log_size) * 100,,0)
      Say ''
      Say 'The connection log file is at' log_file_pct || '% of the maximum'
      Say 'size,' Format(dialer_log_size,,0) 'bytes, specified in the Dialer settings.'
      If log_file_pct > warn_pct Then Do
         Say ''
         Say 'You may want to consider running INETLOG with the'
         Say 'CONDENSE option or increasing the maximum.'
         Say ''
         End
      End
   If \ quiet Then answer = Say_message('Press any key to exit...')
   Else Call SysSleep 2
   Call SysDropFuncs
   save_path = Directory(save_path)
   Exit
   Return  /* for Cleanup */

Initialize_ini:
   Procedure Expose ini_file ini_key. ini_key_value. ini_keys install_path
   trace_initialize_ini = Trace()
   Trace 'N'
   Parse Arg what_r_we, version

   ini_key. = ''
   ini_key_value. = ''
   ini_keys. = ''
   ini_keys = 'OutputFileName inetlog.log',
              'IniFileName Unknown',
              'SummaryFileName inetsum.log',
              'DataPath Unknown',
              'DisplayDailyReport 0',
              'PauseAsScreenFills 1',
              'WarnPercent 85',
              'ConnectDigits 4',
              'MinuteDigits 7',
              'HourDigits 6',
              'LastTimeStamp T00000000000000'  /* the T is to make the field alpha */

   ini_rc = SysIni(ini_file, 'INETLOG', 'ALL:', 'ini_key.')
   If ini_rc <> 'ERROR:' Then Do
      Do i = 1 To ini_key.0
         ini_key_value.i = SysIni(ini_file, 'INETLOG', ini_key.i)
         End
      End
   Else Do
      ini_keys.0 = Words(ini_keys)
      Do i = 1 to ini_keys.0
         ini_keys.i = Word(ini_keys, i)
         End
      Call SysCls
      Say what_r_we
      Say 'Building' ini_file
      Do i = 1 To ini_keys.0 By 2
         j = i + 1
         If ini_keys.i = 'IniFileName' Then Do
            Say 'Please select your dialer by number'
            Say '   1 - DIALER.EXE (1.67 and below)'
            Say '   2 - TCPDIAL.EXE (1.69 and above)'
            Parse Upper Pull ans .
            If ans = 1 Then ini_keys.j = 'dialer.ini'
            If ans = 2 Then ini_keys.j = 'tcpdial.ini'
            If ini_keys.j = 'Unknown' Then Do
               rc = SysFileDelete(ini_file)
               Say 'You must chose a supported dialer, please run again'
               Exit
               End
            End
         Call Ini_update ini_keys.i, ini_keys.j
         End
      Call Initialize_ini version
      Say 'The defaults have been saved in' ini_file || '.'
      Call SysSleep 5
      End

   /* Get path for connection log file and install directory */
   x = Setlocal()
   tcpip_etc_path = Value('etc',,'OS2ENVIRONMENT')
   tcpip_etc_path = tcpip_etc_path || '\'
   x = Endlocal()
   dialer_ini_name = SysIni(ini_file, 'INETLOG', 'IniFileName')
   dialer_ini = tcpip_etc_path || dialer_ini_name
   Parse Value Log_File_Parms(dialer_ini_name, dialer_ini) , 
      With dialer_log_file dialer_log_size

   /* Update data path */
   data_path = Sysini(ini_file, 'INETLOG', 'DataPath')
   If data_path = 'Unknown' | data_path = 'ERROR:' Then Do
      Call Ini_update 'DataPath', install_path || 'DATA\'
      End

   Trace (trace_initialize_ini)
   Return tcpip_etc_path dialer_log_file dialer_log_size install_path data_path

Log_File_Parms:
   Procedure
   trace_initialize_ini = Trace()
   Trace 'N'
   Parse Arg dialer_ini_name, dialer_ini 
   Select
      When Translate(dialer_ini_name) = 'DIALER.INI' Then Do
         dlog_eparm = 'AdvLog'
         dlog_nparm = 'Cfn'
         dlog_sparm = 'Cfs'
         End
      When Translate(dialer_ini_name) = 'TCPDIAL.INI' Then Do
         dlog_eparm = 'Common'
         dlog_nparm = 'LoggingCfn'
         dlog_sparm = 'LoggingCfs'
         End
      Otherwise Do
         Say 'Unable to recognize dialer ini file'
         Call Cleanup
         Exit
         End
      End
   dialer_log_file = Strip(SysIni(dialer_ini, dlog_eparm, dlog_nparm),,'00'x)
   dialer_log_size = C2d(Left(SysIni(dialer_ini, dlog_eparm, dlog_sparm),3))
   Trace (trace_initialize_ini)
   Return dialer_log_file dialer_log_size

Ini_update:
   Procedure Expose ini_file
   Parse Arg name, value
   answer = ''
   Say 'Setting' name 'in INETLOG.INI to' value
   ini_error = SysIni(ini_file, 'INETLOG', name, value)
   If ini_error = '' Then answer = value
   Return answer

Initialize_parms:
   trace_initialize_parms = Trace()
   Trace 'N'
   ModifyLogFile = ''
   quiet = 0
   Parse Value '' With i_parm l_parm o_parm m_parm s_parm d_parm p_parm w_parm

   Do i = 1 to Words(OtherParms)
      Parse Value Word(OtherParms,i) With key_word '=' key_value
      Select
         When Abbrev(key_word,'CON') & ModifyLogFile = '' Then ModifyLogFile = 'CONDENSE'
         When Abbrev(key_word,'Q') Then quiet = 1
         When Abbrev(key_word,'I') Then i_parm = key_value
         When Abbrev(key_word,'L') Then l_parm = key_value
         When Abbrev(key_word,'O') Then o_parm = key_value
         When Abbrev(key_word,'DATAPATH') Then m_parm = key_value
         When Abbrev(key_word,'S') Then s_parm = key_value
         When Abbrev(key_word,'D') Then d_parm = key_value
         When Abbrev(key_word,'P') Then p_parm = key_value
         When Abbrev(key_word,'W') Then w_parm = key_value
         When Abbrev(key_word,'EOF') & ModifyLogFile = '' Then ModifyLogFile = 'EOF2CRLF'
         Otherwise Nop
         End
      End

   If i_parm <> '' Then Do
      Parse Value Update_ini('IniFileName', i_parm) With dialer_ini_name
      dialer_ini = tcpip_etc_path || dialer_ini_name
      Parse Value Log_File_Parms(dialer_ini_name, dialer_ini) , 
         With dialer_log_file dialer_log_size
      End
   If l_parm = '' Then log_file = dialer_log_file
   Else log_file = l_parm
   Parse Value Update_ini('OutputFileName', o_parm) With output_file
   Parse Value Update_ini('SummaryFileName', s_parm) With summary_file
   If m_parm <> '' Then Do
      If Lastpos('\',m_parm) <> Length(m_parm) Then m_parm = m_parm || '\'
      If \ File_cmd(m_parm || '*.*', 'X') Then Do
         Say m_parm 'is not a valid directory, reseting to' install_path || 'DATA\'
         m_parm = install_path || 'DATA\'
         End
      End
   Parse Value Update_ini('DataPath', m_parm) With data_path
   Parse Value Update_ini('DisplayDailyReport', Abbrev(d_parm,'Y')) With daily_report
   Parse Value Update_ini('PauseAsScreenFills', \ Abbrev(p_parm,'N')) With one_screen_atatime
   Parse Value Update_ini('WarnPercent', w_parm) With warn_pct

   /* Leading digits for connects, minutes, hours per day or month.
      You should not need to change these.  If you exceed 9,999
      connects, 9,999,999 minutes, or 999,999 hours per period
      you will crash, but then I think you have other problems. */

   Parse Value Update_ini('ConnectDigits') With sig_x
   /* Significant figures for connects/day or mo */
   Parse Value Update_ini('MinuteDigits') With sig_min
   /* ... for minutes/day or mo */
   Parse Value Update_ini('HourDigits') With sig_hr
   /* ... for hours/day or month */

   Parse Value Update_ini('LastTimeStamp') With last_time_stamp

   /* Get full paths for output file and summary file */
   output_file = data_path || output_file
   summary_file = data_path || summary_file

   Call Set_monthly_summary

   Call Initialize_variables

   Call Prompting

   Trace (trace_initialize_parms)
   Return  /* Initialize_parms */

Set_monthly_summary:  /* set monthly summary */
   monthly. = ''
   summary. = ''
   s_time = 'T00000000000000'
   If File_cmd(summary_file,'R') Then Do
      Do i = 1 While Lines(summary_file) > 0
         data = Linein(summary_file)
         If \ Abbrev(data,'*') & data <> '' Then Do
            Parse Var data acctid userid '*' s_yr s_mo s_time s_data
            If s_yr = '' Then Do
               acctid = 'unknown'
               userid = 'unknown'
               Parse Var data s_yr s_mo s_data
               End
            acctid = Strip(acctid)
            userid = Strip(userid)
            s_yr = Strip(s_yr)
            s_mo = Strip(s_mo)
            monthly.i = acctid userid s_yr s_mo
            summary.acctid.userid.s_yr.s_mo = s_yr s_mo  s_time s_data
            End
         Else monthly.i = data
         End
      Call File_cmd summary_file, 'C'
      monthly.0 = i - 1
      End
   Else Do
      Say summary_file 'not found . . . will try to build'
      summary_file_sample = Left(summary_file,Lastpos('.',summary_file) -1) || '.smp'
      If File_cmd(summary_file_sample,'X') Then Do
         '@COPY' summary_file_sample summary_file '> NUL'
         Call Set_monthly_summary
         s_time = 'T00000000000000' /* reset timestamp to get updates */
         End
      Else Do
         Say 'Sample summary file' summary_file_sample 'not found . . . Aborting'
         Call Cleanup
         End
      End
   last_time_stamp = s_time
   Return /* Set_monthly_summary */

Initialize_variables:
   crlf = D2c(13) || D2c(10)  /* carriage return + linefeed */
   esc = D2c(27)              /* Escape character */
   time_stamp = ''            /* Time stamp of each connect record */

   Do i = 1 To 12  /* initialize months */
      x = Right(i,2,'0')
      month.x = Word('Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec',i)
      End

   /* calculated variables
      signons_each_day    Accumulate number of connects daily
      signons_each_month  Accumulate number of connects monthly
      time_on             Time, each connect  (minutes) 
      daily_time_on       Accumulated minutes, daily
      monthly_time_on     Accumulated minutes, monthly */ 

   /* More variables: these we initialize as follows: */
   old_m = 0       /* Storage of a 2-digit month (eg 05 = May) */
   old_d = 0       /* ... and a 2-digit day */ 
   dcounter = 0    /* Counter increments each sign-on in a day */
   mcounter = 0	   /* Same for each sign-on in a month */
   monthline = ''  /* Initialize both of these as null strings */
   dayline = ''    /* These are for monthly and daily output strings */

   /* A typical line generated in the connect.log upon disconnect looks like
      either this for a dialer version preceding v1.45:
         11/15 19:08:38 Disconnected after 00:06:20  0 errors  0 discards'
      or this style for v. 1.45 and later:
         1995/11/15 19:08:38 Disconnected after 00:06:20  0 errors  0 discards
      We search for a key_phrase using the RexxUtil function SysFileSearch */

   key_phrase = 'Disconnected after' /* Word or phrase we'll search for */
   user_phrase = 'dialed' /* word to search for account and userid */

   Return  /* Initialize_variables */

Prompting:  /* Initialize and do other input/output file stuff */

   If ModifyLogFile = 'CONDENSE',
      | ModifyLogFile = 'EOF2CRLF' Then Do
      output_file_bak = output_file  /* Back up output_file */
      Parse Var log_file fname '.' ext
      output_file = fname||'.new'  /* Name for a connect.new file we will make */
      /* Erase any instance of this new file if it exists */
      If File_cmd(output_file,'X') Then Call SysFileDelete output_file
      End


   /* Here is where we Check if the output file exists.  If it does, we
      overwrite it, and if not we create it.  BUT....  we don't want to 
      do something stupid like try to erase a vital file or the connect 
      logfile... */

   If \ File_cmd(log_file,'X') Then Do
      Say 'Aborting.' log_file 'does not exist.'
      Say 'Enter l=correct logfile name as a parm in INETLOG.CMD.'
      Call Beep 1000,100
      Call Cleanup
      End

   file1 = Translate(Filespec('name', log_file))
   file2 = Translate(Filespec('name', output_file))
   If file2 == 'CONNECT.LOG',
      | file2 == 'CONFIG.SYS',
      | file2 == 'AUTOEXEC.BAT',
      | file2 == file1 Then Do
      Say Translate(log_file) 'and' Translate(output_file)
      Say 'were just entered as logfile and output file, respectively.'
      Say 'CONNECT.LOG, CONFIG.SYS, AUTOEXEC.BAT, or the name you select'
      Say 'as the logfile, are all not permitted as output filenames.'
      Say crlf
      Call Beep 1000,100
      Call Cleanup  /* Error, so Exit */
      End

   /* make sure that we have control of the log file */
   rc = File_cmd(log_file,'W')
   If \ rc Then Do Until rc
      Say 'Log file' log_file 'returned' result
      rc = File_cmd(log_file,'C')
      Say 'Unable to open' log_file
      Say 'Press Esc to abort run'
      answer = Say_message('Press any other key to wait 5 seconds')
      If answer = esc Then Call Cleanup
      Call SysSleep 5
      Call SysCls
      rc = File_cmd(log_file,'W')
      End
   rc = File_cmd(log_file,'C')

   /* Backup any output file of the same name if it exists, then erase orig. */
   If File_cmd(output_file,'X') Then Do
      Parse Var output_file fname '.' ext
      '@COPY' output_file fname||'.bak > NUL'
      Call SysFileDelete output_file
      End

   Return /* Prompting */

Update_ini:
   Procedure Expose ini_file ini_key. ini_key_value. ini_keys
   Parse Arg name, value
   trace_update_ini = Trace()
   Trace 'N'

   Select
      When name = 'WarnPercent' Then Do
         If value = '' Then value = 85
         Else Do
            If Datatype(value,'NUM'),
               & value >= 50 & value <= 95 Then value = Format(value,,0)
            Else Do
               bad = Format(value,,0)
               value = Word(ini_keys, Wordpos(name,ini_keys) + 1)
               Say 'The w= parameter must be between 50 and 95,'
               Say 'you entered w=' || bad || '(or null), w=' || value 'assumed.'
               answer = Say_message('Press any key to continue...')
               End
            End
         End
      Otherwise Nop
      End
   If value = '' Then answer = Ini_value(name)
   Else Do
      If value = Ini_value(name) Then answer = value
      Else answer = Ini_update(name, value)
      End
   If answer = 'ERROR:' Then Do
      value = Word(ini_keys,Wordpos(name,ini_keys) + 1)
      ini_error = Ini_update(name, value)
      If ini_error = '' Then answer = value
      End

   Trace (trace_update_ini)
   Return answer

Ini_value:
   Procedure Expose ini_key. ini_key_value.
   Parse Arg ini_key
   Do i = 1 to ini_key.0 + 1 Until ini_key = ini_key.i
      End
   ini_key_value = ini_key_value.i
   Return ini_key_value

File_cmd:  /* performs common Stream commands and returns 1 if successful */
   Procedure Expose result
   trace_file_cmd = Trace()
   Trace 'N'
   Parse Arg file_name, command
   command = Translate(command)
   Select
      When command = 'X' Then action = 'QUERY EXISTS'
      When command = 'C' Then action = 'CLOSE'
      When command = 'W' Then action = 'OPEN WRITE'
      When command = 'R' Then action = 'OPEN READ'
      Otherwise Return 0
      End
   result = Stream(file_name, 'C', action)
   Select
      When command = 'X' Then answer = (result <> '')
      When command = 'C' Then answer = Abbrev(result,'READY') | (result = '')
      Otherwise answer = Abbrev(result,'READY')
      End
   Trace (trace_file_cmd)
   Return answer

Say_out:  /* performs output to the screen */
   Procedure Expose quiet
   If \ quiet Then Do
      Parse Arg line
      Say line
      End
   Return

Say_message:  /* performs message output and returns key entered */
   Procedure Expose quiet
   Parse Arg msg
   Say msg
   answer = SysGetKey('NOECHO')
   Return answer

Fix_eof:  /* This routine will copy the log file and replace any EOF
             characters found with CRLF.  It is required If the log file 
             has been edited.  */

   log_file_bak = log_file			/* Save this filespec */
   eof_found = Eof2crlf(log_file output_file)	/* Remove any EOF characters */
   log_file = output_file			/* Use output_file that we just created from Fix_Log as the log file */
   output_file = output_file_bak		/* Restore this as file to output to */
   ModifyLogFile = 'TESTFIXEDLOG'		/* Run the main routine to do test new connection log */
   Call SysCls
   Say 'The conversion is done.  There were' eof_found 'EOF characters in' log_file_bak
   Say crlf
   Say log_file 'is the corrected connection log file.'
   Say 'We will now test it.  Please evaluate whether'
   Say 'the data being spit out is reasonable.  The output will'
   Say 'also be saved to disk as' output_file
   Say 'which can be examined with the System Editor.'
   Say 'If you want to quit now, press ESCAPE.'
   answer = Say_message('Otherwise, press any other key to run the test.')
   If answer = esc Then Call Cleanup
   Call SysCls
   Say 'Starting the analysis of' log_file 'as the log file.'
   Call SysSleep 2				/* Let us read the message */

   /* Back up any output file of the same name if it exists, then erase orig. */
   If File_cmd(output_file,'X') Then Do
      Parse Var output_file fname '.' ext
      '@COPY' output_file fname || '.bak > NUL'
      Call SysFileDelete output_file
      End

   Return /* Return from Fix_eof */

Eof2crlf:
   Procedure
   /* EOF2CRLF.CMD		 Jerry Levy	18 Mar 95
      Replaces E-O-F's (26 decimal or 1A hex or Control-Z)
      by a carriage return+linefeed
      Syntax:
         eof2crlf inputfile outputfile */

   eof = D2c(26)
   cr = D2c(13)
   lf = D2c(10)
   found = 0

   Parse Arg inputfile outputfile
   Call File_cmd inputfile, 'R'
   Call File_cmd outputfile, 'W'

   Do While Chars(inputfile) > 0
      datachar = CharIn(inputfile)
      If datachar = eof Then Do
         found = found + 1
         datachar = cr
         Call Charout outputfile, datachar
         datachar = lf
         End
      Call Charout outputfile, datachar
      End
   Call File_cmd inputfile, 'C'
   Call File_cmd outputfile, 'C'
   Return found

Log_file_cleanup:
   rc = File_cmd(log_file,'C')
   rc = File_cmd(output_file,'C')
   Say 'If everything looks OK, pressing R will rename'
   Say log_file 'to' log_file_bak

   Parse Var log_file_bak fname '.' ext
   '@COPY' log_file_bak  fname || '.bak > NUL'
 
   Say 'The original log file is already backed up'
   Say 'as' fname || '.bak' crlf
   If Translate(Say_message('               Press R to Rename or any other key to exit...')) = 'R' Then Do

   /* Remember: log_file_bak is the original connect logfile name
      and log_file is now the .NEW connect log we just created */
      '@COPY' log_file log_file_bak '> NUL'
      If File_cmd(log_file,'X') Then Call SysFileDelete log_file
      End

   Return  /* from Log_file_cleanup */
